require "rpi_union/version"
require 'httparty'

module RpiUnion
  class Client
    include HTTParty

    API_BASE_URI = 'http://api.union.rpi.edu/query.php'
    base_uri API_BASE_URI

    GET_USER_TASK = 'GetUser'
    LIST_ALL_ORGANIZATIONS_TASK = 'ListAllOrganizations'
    GET_ORGANIZATION_TASK = 'GetOrganization'
    GET_USER_ORGS_TASK = 'GetUserOrgs'

    # Initialize a new Client with the specified API key
    def initialize(api_key)
      @api_key = api_key
    end

    # Get the properties of a specific user.
    #
    # @param options [Hash] Must contain one of: `:rin`, `:id` (API/database ID), or `rcsid`
    # @return [Hash] The output of the API call.  This is of the form:
    #   `{"name": ..., "rin": ..., "phone": ..., "class": ...}`
    def get_user(options = {})
      opts = {}
      opts[:query] = {}

      if options.key?(:rin)
        opts[:query][:rin] = options[:rin]
      elsif options.key?(:rcsid)
        opts[:query][:rcsid] = options[:rcsid]
      elsif options.key?(:id)
        opts[:query][:dbid] = options[:id]
      else
        raise "You must pass one of the following as an option to get_user(): "+
          "'rin', 'rcsid', or 'id'"
      end

      # Set middlename if it was pass in options
      if options.key?(:middlename)
        opts[:query][:middlename] = options[:middlename]
      end

      get_with_key_task(GET_USER_TASK, opts)
    end

    # Get all of the Union clubs
    #
    # @return [Array] An array of hashes of the form `{"club_id": ..., "club_name": ...}`
    def list_all_organizations
      get_with_key_task(LIST_ALL_ORGANIZATIONS_TASK)
    end

    # Get information about a club
    #
    # @param id [Int] The ID of the club
    # @return [Hash] The result of the API call.  Send an example request to see the format.
    def get_organization(id)
      opts = { :query => { :id => id } }
      get_with_key_task(GET_ORGANIZATION_TASK, opts)
    end

    # Get the IDs of all the clubs of which the user is an officer
    #
    # @param rin [Int] The RIN of the user to search for.
    def get_user_orgs(rin)
      opts = { :query => { :rin => rin } }
      get_with_key_task(GET_USER_ORGS_TASK, opts)
    end

    private

    def get_with_key_task(task, options = {})
      # Add the api key to the request
      options[:query] ||= {}
      options[:query].merge!({ :apikey => @api_key, :task => task })

      res = self.class.get('', options)
      data = JSON.parse(res.body)

      raise ApiError.new(data['status']) unless data['status']['statusCode'] == 0

      data['result']
    end
  end

  class ApiError < StandardError
    ERROR_TYPES = {
      2 => "Task Not Found",
      3 => "Database Error",
      4 => "Invalid API Key",
      5 => "Unauthorized Origin",
      10 => "Validation Error",
      11 => "Data Not Found",
    }

    def initialize(status_obj)
      @code = status_obj['code']
      @text = status_obj['text']
    end

    def get_error_type(code)
      return "API Error" unless ERROR_TYPES.key?(code)
      ERROR_TYPES[code]
    end

    def message
      return "#{get_error_type(@code)}: #{@text}"
    end
  end
end
